"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.SuspendAutoScalingGroupsTrick = void 0;
const tslib_1 = require("tslib");
const aws_sdk_1 = tslib_1.__importDefault(require("aws-sdk"));
const chalk_1 = tslib_1.__importDefault(require("chalk"));
class SuspendAutoScalingGroupsTrick {
    constructor() {
        this.asClient = new aws_sdk_1.default.AutoScaling();
    }
    getMachineName() {
        return SuspendAutoScalingGroupsTrick.machineName;
    }
    async getCurrentState(task, currentState, options) {
        const scalingGroups = await this.listAutoScalingGroups(task);
        if (!scalingGroups || scalingGroups.length === 0) {
            task.skip(chalk_1.default.dim('no ASG found'));
            return;
        }
        currentState.push(...scalingGroups.map((asg) => {
            return {
                name: asg.AutoScalingGroupName,
            };
        }));
    }
    async conserve(task, currentState, options) {
        const subListr = task.newListr([], {
            concurrent: 10,
            exitOnError: false,
            rendererOptions: {
                collapse: true,
            },
        });
        if (currentState && currentState.length > 0) {
            for (const asgState of currentState) {
                subListr.add({
                    title: chalk_1.default.blue(`${asgState.name}`),
                    task: (ctx, task) => this.conserveProcesses(task, asgState, options),
                    options: {
                        persistentOutput: true,
                    },
                });
            }
        }
        else {
            task.skip(chalk_1.default.dim(`no auto scaling groups found`));
        }
        return subListr;
    }
    async restore(task, originalState, options) {
        const subListr = task.newListr([], {
            concurrent: 10,
            exitOnError: false,
            rendererOptions: {
                collapse: true,
            },
        });
        if (originalState && originalState.length > 0) {
            for (const asgState of originalState) {
                subListr.add({
                    title: chalk_1.default.blue(`${asgState.name}`),
                    task: (ctx, task) => this.restoreProcesses(task, asgState, options),
                    options: {
                        persistentOutput: true,
                    },
                });
            }
        }
        else {
            task.skip(chalk_1.default.dim(`no auto scaling groups were conserved`));
        }
        return subListr;
    }
    async listAutoScalingGroups(task) {
        const groups = [];
        // TODO Add logic to go through all pages
        task.output = 'fetching page 1...';
        groups.push(...(await this.asClient
            .describeAutoScalingGroups({ MaxRecords: 100 })
            .promise()).AutoScalingGroups);
        task.output = 'done';
        return groups;
    }
    async conserveProcesses(task, asgState, options) {
        if (options.dryRun) {
            task.skip(chalk_1.default.dim('skipped, would suspend ASG processes'));
            return;
        }
        task.output = 'suspending ASG processes...';
        await this.asClient
            .suspendProcesses({
            AutoScalingGroupName: asgState.name,
        })
            .promise();
        task.output = 'suspended';
    }
    async restoreProcesses(task, asgState, options) {
        if (options.dryRun) {
            task.skip(chalk_1.default.dim(`skipped, would resume ASG processes`));
            return;
        }
        task.output = 'resuming ASG...';
        await this.asClient
            .resumeProcesses({
            AutoScalingGroupName: asgState.name,
        })
            .promise();
        task.output = `Restored`;
    }
}
exports.SuspendAutoScalingGroupsTrick = SuspendAutoScalingGroupsTrick;
SuspendAutoScalingGroupsTrick.machineName = 'suspend-auto-scaling-groups';
//# sourceMappingURL=suspend-auto-scaling-groups.trick.js.map